@@ -10,14 +10,22 @@ import asyncio
 import json
 import logging
 import re
-from typing import Any, List, Optional
 
-from .const import __dev_mode__  # DEVICE_ID_REGEX,
-from .protocol import RAMSES_CODES, Command, ExpiredCallbackError, Priority
+from .const import SZ_SCHEDULE, SZ_ZONE_IDX, __dev_mode__
+from .protocol import CODES_SCHEMA, Command, ExpiredCallbackError, Priority
+from .protocol.command import _mk_cmd
 from .protocol.opentherm import OTB_MSG_IDS
 
-from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
-from .protocol import (  # noqa: F401, isort: skip
+# skipcq: PY-W2000
+from .protocol import (  # noqa: F401, isort: skip, pylint: disable=unused-import
+    I_,
+    RP,
+    RQ,
+    W_,
+)
+
+# skipcq: PY-W2000
+from .protocol import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _0001,
     _0002,
     _0004,
@@ -47,6 +55,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _10E0,
     _10E1,
     _1100,
+    _11F0,
     _1260,
     _1280,
     _1290,
@@ -60,6 +69,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _1F09,
     _1F41,
     _1FC9,
+    _1FCA,
     _1FD0,
     _1FD4,
     _2249,
@@ -77,7 +87,9 @@ from .protocol import (  # noqa: F401, isort: skip
     _2420,
     _2D49,
     _2E04,
+    _2E10,
     _30C9,
+    _3110,
     _3120,
     _313F,
     _3150,
@@ -121,20 +133,20 @@ if DEV_MODE:
 
 
 def script_decorator(fnc):
-    def wrapper(gwy, *args, **kwargs) -> Optional[Any]:
+    def wrapper(gwy, *args, **kwargs):
 
-        gwy.send_cmd(Command._puzzle(message="Script begins:", **QOS_DEFAULT_HIGH))
+        gwy.send_cmd(Command._puzzle(message="Script begins:", qos=QOS_DEFAULT_HIGH))
 
         result = fnc(gwy, *args, **kwargs)
 
-        gwy.send_cmd(Command._puzzle(message="Script done.", **QOS_DEFAULT_LOW))
+        gwy.send_cmd(Command._puzzle(message="Script done.", qos=QOS_DEFAULT_LOW))
 
         return result
 
     return wrapper
 
 
-def spawn_scripts(gwy, **kwargs) -> List[asyncio.Task]:
+def spawn_scripts(gwy, **kwargs) -> list[asyncio.Task]:
 
     tasks = []
 
@@ -168,7 +180,7 @@ async def exec_cmd(gwy, **kwargs):
     #     _LOGGER.warning("Execute: Command is invalid: '%s'", kwargs)
     #     return
 
-    cmd = Command.from_str(kwargs[EXEC_CMD], **QOS_DEFAULT_HIGH)
+    cmd = Command.from_cli(kwargs[EXEC_CMD], qos=QOS_DEFAULT_HIGH)
 
     await gwy.async_send_cmd(cmd)
 
@@ -190,16 +202,16 @@ async def periodic(gwy, cmd, count=1, interval=None):
 
 
 async def get_faults(gwy, ctl_id: str, start=0, limit=0x3F):
-    device = gwy._get_device(ctl_id, ctl_id=ctl_id)
+    ctl = gwy.get_device(ctl_id)
 
     try:
-        await device._evo.get_faultlog(start=start, limit=limit)  # 0418
+        await ctl.tcs.get_faultlog(start=start, limit=limit)  # 0418
     except ExpiredCallbackError as exc:
         _LOGGER.error("get_faults(): Function timed out: %s", exc)
 
 
 async def get_schedule(gwy, ctl_id: str, zone_idx: str) -> None:
-    zone = gwy._get_device(ctl_id, ctl_id=ctl_id)._evo._get_zone(zone_idx)
+    zone = gwy.get_device(ctl_id).tcs.get_htg_zone(zone_idx)
 
     try:
         await zone.get_schedule()
@@ -209,31 +221,31 @@ async def get_schedule(gwy, ctl_id: str, zone_idx: str) -> None:
 
 async def set_schedule(gwy, ctl_id, schedule) -> None:
     schedule = json.load(schedule)
-    zone_idx = schedule["zone_idx"]
+    zone_idx = schedule[SZ_ZONE_IDX]
 
-    zone = gwy._get_device(ctl_id, ctl_id=ctl_id)._evo._get_zone(zone_idx)
+    zone = gwy.get_device(ctl_id).tcs.get_htg_zone(zone_idx)
 
     try:
-        await zone.set_schedule(schedule["schedule"])  # 0404
+        await zone.set_schedule(schedule[SZ_SCHEDULE])  # 0404
     except ExpiredCallbackError as exc:
         _LOGGER.error("set_schedule(): Function timed out: %s", exc)
 
 
 async def script_bind_req(gwy, dev_id: str):
-    gwy._get_device(dev_id)._make_fake(bind=True)
+    gwy.get_device(dev_id)._make_fake(bind=True)
 
 
 async def script_bind_wait(gwy, dev_id: str, code=_2309, idx="00"):
-    gwy._get_device(dev_id)._make_fake(bind=True, code=code, idx=idx)
+    gwy.get_device(dev_id)._make_fake(bind=True, code=code, idx=idx)
 
 
-def script_poll_device(gwy, dev_id) -> List[Any]:
+def script_poll_device(gwy, dev_id) -> list:
     _LOGGER.warning("poll_device() invoked...")
 
     tasks = []
 
     for code in (_0016, _1FC9):
-        cmd = Command(RQ, code, "00", dev_id, **QOS_DEFAULT)
+        cmd = _mk_cmd(RQ, code, "00", dev_id, **QOS_DEFAULT)
         tasks.append(gwy._loop.create_task(periodic(gwy, cmd, count=0)))
 
     gwy._tasks.extend(tasks)
@@ -244,7 +256,7 @@ def script_poll_device(gwy, dev_id) -> List[Any]:
 async def script_scan_disc(gwy, dev_id: str):
     _LOGGER.warning("scan_quick() invoked...")
 
-    gwy._get_device(dev_id)._discover()  # discover_flag=Discover.ALL)
+    gwy.get_device(dev_id)._discover()  # discover_flag=Discover.DEFAULT)
 
 
 @script_decorator
@@ -252,25 +264,25 @@ async def script_scan_full(gwy, dev_id: str):
     _LOGGER.warning("scan_full() invoked - expect a lot of Warnings")
 
     qos = {"priority": Priority.DEFAULT, "retries": 5}
-    gwy.send_cmd(Command(RQ, _0016, "0000", dev_id, **qos))
+    gwy.send_cmd(_mk_cmd(RQ, _0016, "0000", dev_id, **qos))
 
     qos = {"priority": Priority.DEFAULT, "retries": 1}
-    for code in sorted(RAMSES_CODES):
+    for code in sorted(CODES_SCHEMA):
         if code == _0005:
             for zone_type in range(20):  # known up to 18
-                gwy.send_cmd(Command(RQ, code, f"00{zone_type:02X}", dev_id, **qos))
+                gwy.send_cmd(_mk_cmd(RQ, code, f"00{zone_type:02X}", dev_id, **qos))
 
         elif code == _000C:
             for zone_idx in range(16):  # also: FA-FF?
-                gwy.send_cmd(Command(RQ, code, f"{zone_idx:02X}00", dev_id, **qos))
+                gwy.send_cmd(_mk_cmd(RQ, code, f"{zone_idx:02X}00", dev_id, **qos))
 
         elif code == _0016:
             continue
 
         # elif code in (_01D0, _01E9):
-        #     for zone_idx in ("00", "01", "99", "FC", "FF"):
-        #         gwy.send_cmd(Command(W_, code, f"{zone_idx}00", dev_id, **qos))
-        #         gwy.send_cmd(Command(W_, code, f"{zone_idx}03", dev_id, **qos))
+        #     for zone_idx in ("00", "01", "99", FC, FF):
+        #         gwy.send_cmd(_mk_cmd(W_, code, f"{zone_idx}00", dev_id, **qos))
+        #         gwy.send_cmd(_mk_cmd(W_, code, f"{zone_idx}03", dev_id, **qos))
 
         elif code == _0404:
             gwy.send_cmd(Command.get_schedule_fragment(dev_id, "HW", 0, 0, **qos))
@@ -294,19 +306,19 @@ async def script_scan_full(gwy, dev_id: str):
             continue
 
         elif (
-            code in RAMSES_CODES
-            and RQ in RAMSES_CODES[code]
-            and re.match(RAMSES_CODES[code][RQ], "00")
+            code in CODES_SCHEMA
+            and RQ in CODES_SCHEMA[code]
+            and re.match(CODES_SCHEMA[code][RQ], "00")
         ):
-            gwy.send_cmd(Command(RQ, code, "00", dev_id, **qos))
+            gwy.send_cmd(_mk_cmd(RQ, code, "00", dev_id, **qos))
 
         else:
-            gwy.send_cmd(Command(RQ, code, "0000", dev_id, **qos))
+            gwy.send_cmd(_mk_cmd(RQ, code, "0000", dev_id, **qos))
 
     # these are possible/difficult codes
     qos = {"priority": Priority.DEFAULT, "retries": 2}
     for code in (_0150, _2389):
-        gwy.send_cmd(Command(RQ, code, "0000", dev_id, **qos))
+        gwy.send_cmd(_mk_cmd(RQ, code, "0000", dev_id, **qos))
 
 
 @script_decorator
@@ -317,7 +329,7 @@ async def script_scan_hard(gwy, dev_id: str):
     #     nonlocal code
 
     #     if (code := code + 1) < 0x4000:
-    #         cmd = Command(RQ, f"{code:04X}", "0000", dev_id, **QOS_DEFAULT_SCAN)
+    #         cmd = _mk_cmd(RQ, f"{code:04X}", "0000", dev_id, **QOS_DEFAULT_SCAN)
     #         # gwy.msg_protocol.send_data(cmd, callback=callback)
     #         gwy.send_cmd(cmd, callback=callback)
 
@@ -325,7 +337,7 @@ async def script_scan_hard(gwy, dev_id: str):
     # callback(None)
 
     for code in range(0x4000):
-        gwy.send_cmd(Command(RQ, f"{code:04X}", "0000", dev_id, **QOS_DEFAULT_SCAN))
+        gwy.send_cmd(_mk_cmd(RQ, f"{code:04X}", "0000", dev_id, **QOS_DEFAULT_SCAN))
         await asyncio.sleep(1)
 
 
@@ -335,21 +347,21 @@ async def script_scan_001(gwy, dev_id: str):
 
     qos = {"priority": Priority.LOW, "retries": 3}
     for idx in range(0x10):
-        gwy.send_cmd(Command(W_, _000E, f"{idx:02X}0050", dev_id, **qos))
-        gwy.send_cmd(Command(RQ, _000E, f"{idx:02X}00C8", dev_id, **qos))
+        gwy.send_cmd(_mk_cmd(W_, _000E, f"{idx:02X}0050", dev_id, **qos))
+        gwy.send_cmd(_mk_cmd(RQ, _000E, f"{idx:02X}00C8", dev_id, **qos))
 
 
 @script_decorator
 async def script_scan_002(gwy, dev_id: str):
     _LOGGER.warning("scan_002() invoked - expect a lot of nonsense")
 
-    # Two modes, I and W & Two headers zz00 and zz
+    # Two modes, I & W, and Two contexts, zz & zz00
     message = "0000" + "".join(f"{ord(x):02X}" for x in "Hello there.") + "00"
 
     [
-        gwy.send_cmd(Command(W_, f"{c:04X}", message, dev_id, **QOS_DEFAULT))
+        gwy.send_cmd(_mk_cmd(W_, f"{c:04X}", message, dev_id, **QOS_DEFAULT))
         for c in range(0x4000)
-        if c not in RAMSES_CODES
+        if c not in CODES_SCHEMA
     ]
 
 
@@ -386,16 +398,17 @@ async def script_scan_otb_map(gwy, dev_id: str):  # Tested only upon a R8820A
         _22D9: "01",  # boiler setpoint        / ControlSetpoint
         _3EF1: "11",  # rel. modulation level  / RelativeModulationLevel
         _1300: "12",  # cv water pressure      / CHWaterPressure
+        _12F0: "13",  # dhw_flow_rate          / DHWFlowRate
         _3200: "19",  # boiler output temp     / BoilerWaterTemperature
         _1260: "1A",  # dhw temp               / DHWTemperature
         _1290: "1B",  # outdoor temp           / OutsideTemperature
         _3210: "1C",  # boiler return temp     / ReturnWaterTemperature
-        _10A0: "38",  # dhw params["setpoint"] / DHWSetpoint
+        _10A0: "38",  # dhw params[SZ_SETPOINT] / DHWSetpoint
         _1081: "39",  # max ch setpoint        / MaxCHWaterSetpoint
     }
 
     for code, msg_id in RAMSES_TO_OPENTHERM.items():
-        gwy.send_cmd(Command(RQ, code, "00", dev_id, **QOS_DEFAULT))
+        gwy.send_cmd(_mk_cmd(RQ, code, "00", dev_id, **QOS_DEFAULT))
         gwy.send_cmd(Command.get_opentherm_data(dev_id, msg_id, **QOS_DEFAULT))
 
 
@@ -414,14 +427,14 @@ async def script_scan_otb_ramses(gwy, dev_id: str):  # Tested only upon a R8820A
         _2420,
         _1300,  # cv water pressure      / CHWaterPressure
         _1081,  # max ch setpoint        / MaxCHWaterSetpoint
-        _10A0,  # dhw params["setpoint"] / DHWSetpoint
+        _10A0,  # dhw params[SZ_SETPOINT] / DHWSetpoint
         _22D9,  # boiler setpoint        / ControlSetpoint
         _1260,  # dhw temp               / DHWTemperature
         _1290,  # outdoor temp           / OutsideTemperature
         _3200,  # boiler output temp     / BoilerWaterTemperature
         _3210,  # boiler return temp     / ReturnWaterTemperature
         _0150,
-        _12F0,  # DHW flow rate?
+        _12F0,  # dhw flow rate          / DHWFlowRate
         _1098,
         _10B0,
         _3221,
@@ -435,7 +448,7 @@ async def script_scan_otb_ramses(gwy, dev_id: str):  # Tested only upon a R8820A
     #  - ch setpoint          /
     #  - max. rel. modulation /
 
-    [gwy.send_cmd(Command(RQ, c, "00", dev_id, **QOS_DEFAULT)) for c in CODES]
+    [gwy.send_cmd(_mk_cmd(RQ, c, "00", dev_id, **QOS_DEFAULT)) for c in CODES]
 
 
 SCRIPTS = {
