@@ -7,26 +7,50 @@
 
 from logging import Logger
 
-from .address import is_valid_dev_id
+from .address import Address, is_valid_dev_id
 from .command import CODE_API_MAP, Command, FaultLog, Priority
-from .exceptions import CorruptStateError, ExpiredCallbackError, InvalidPacketError
+from .const import (
+    SZ_DEVICE_ROLE,
+    SZ_DOMAIN_ID,
+    SZ_ZONE_CLASS,
+    SZ_ZONE_IDX,
+    SZ_ZONE_MASK,
+    SZ_ZONE_TYPE,
+    __dev_mode__,
+)
+from .exceptions import (
+    CorruptStateError,
+    ExpiredCallbackError,
+    InvalidAddrSetError,
+    InvalidPacketError,
+)
 from .logger import set_logger_timesource, set_pkt_logging
 from .message import Message
 from .packet import _PKT_LOGGER, Packet
 from .protocol import create_msg_stack
-from .ramses import RAMSES_CODES, RAMSES_DEVICES
-from .schedule import Schedule
-from .schema import PACKET_LOG, PACKET_LOG_SCHEMA, SERIAL_PORT, SERIAL_PORT_SCHEMA
-from .transport import POLLER_TASK, create_pkt_stack
+from .ramses import CODES_BY_DEV_SLUG, CODES_SCHEMA
+from .schema import PACKET_LOG, PACKET_LOG_SCHEMA, SERIAL_CONFIG_SCHEMA, SERIAL_PORT
+from .transport import SZ_POLLER_TASK, create_pkt_stack
 
+# skipcq: PY-W2000
 from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     I_,
     RP,
     RQ,
     W_,
-    __dev_mode__,
+    F9,
+    FA,
+    FC,
+    FF,
+    DEV_ROLE,
+    DEV_ROLE_MAP,
+    DEV_TYPE,
+    DEV_TYPE_MAP,
+    ZON_ROLE,
+    ZON_ROLE_MAP,
 )
 
+# skipcq: PY-W2000
 from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _0001,
     _0002,
@@ -57,6 +81,7 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _10E0,
     _10E1,
     _1100,
+    _11F0,
     _1260,
     _1280,
     _1290,
@@ -70,6 +95,7 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _1F09,
     _1F41,
     _1FC9,
+    _1FCA,
     _1FD0,
     _1FD4,
     _2249,
@@ -87,7 +113,9 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _2420,
     _2D49,
     _2E04,
+    _2E10,
     _30C9,
+    _3110,
     _3120,
     _313F,
     _3150,
