@@ -6,25 +6,40 @@
 Provide the base class for commands (constructed/sent packets) and packets.
 """
 
-import logging
-from typing import Optional, Tuple, Union
+# TODO: add _has_idx (ass func return only one type, or raise)
 
-from .address import Address
-from .const import NON_DEVICE_ID, NUL_DEVICE_ID
-from .exceptions import InvalidPayloadError
+import logging
+from typing import Optional, Union
+
+from .address import NON_DEV_ADDR, NUL_DEV_ADDR, Address, pkt_addrs
+from .const import COMMAND_REGEX, DEV_ROLE_MAP, DEV_TYPE_MAP, __dev_mode__
+from .exceptions import InvalidPacketError, InvalidPayloadError
 from .ramses import (
     CODE_IDX_COMPLEX,
     CODE_IDX_DOMAIN,
     CODE_IDX_NONE,
     CODE_IDX_SIMPLE,
-    CODE_ONLY_FROM_CTL,
+    CODES_ONLY_FROM_CTL,
+    CODES_SCHEMA,
     CODES_WITH_ARRAYS,
-    RAMSES_CODES,
     RQ_NO_PAYLOAD,
 )
 
-from .const import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
-from .const import (  # noqa: F401, isort: skip
+# skipcq: PY-W2000
+from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
+    I_,
+    RP,
+    RQ,
+    W_,
+    F8,
+    F9,
+    FA,
+    FC,
+    FF,
+)
+
+# skipcq: PY-W2000
+from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _0001,
     _0002,
     _0004,
@@ -54,6 +69,7 @@ from .const import (  # noqa: F401, isort: skip
     _10E0,
     _10E1,
     _1100,
+    _11F0,
     _1260,
     _1280,
     _1290,
@@ -67,6 +83,7 @@ from .const import (  # noqa: F401, isort: skip
     _1F09,
     _1F41,
     _1FC9,
+    _1FCA,
     _1FD0,
     _1FD4,
     _2249,
@@ -84,7 +101,9 @@ from .const import (  # noqa: F401, isort: skip
     _2420,
     _2D49,
     _2E04,
+    _2E10,
     _30C9,
+    _3110,
     _3120,
     _313F,
     _3150,
@@ -109,70 +128,127 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-class PacketBase:
-    """The packet base - used by Command and Packet classes."""
+class Frame:
+    """The Frame class - used as a base by the Command and Packet classes.
 
-    def __init__(self) -> None:
-        """Create a frame."""
-        self._dtm = None
-        self._frame = None
+    `RQ --- 01:078710 10:067219 --:------ 3220 005 0000050000`
+    """
 
-        self._rssi = None
-        self._verb = None
-        self._seqn = None
-        self._src = None
-        self._dst = None
-        self._addrs = [None] * 3
-        self._code = None
-        self._len = None
-        self._payload = None
+    _frame: str
 
-        self._has_array_ = None
-        self._has_ctl_ = None
-        self._has_payload_ = None
-        self._ctx_ = None
-        self._hdr_ = None
-        self._idx_ = None
+    verb: str
+    seqn: str  # TODO: or, better as int?
+    src: Address
+    dst: Address
+    _addrs: tuple[Address, Address, Address]
+    code: str
+    len_: str
+    _len: int  # int(len(payload) / 2)
+    payload: str
+
+    def __init__(self, frame: str) -> None:
+        """Create a frame from a string.
+
+        Will raise InvalidPacketError if it is invalid.
+        """
+
+        self._frame = frame
+        if not isinstance(self._frame, str):
+            raise InvalidPacketError(f"Bad frame: not a string: {type(self._frame)}")
+        if not COMMAND_REGEX.match(self._frame):
+            raise InvalidPacketError("Bad frame: invalid structure")
+
+        fields = frame.lstrip().split(" ")
+
+        self.verb = frame[:2]  # cant use fields[0] as leading space ' I'
+        self.seqn = fields[1]  # frame[3:6]
+        self.code = fields[5]  # frame[37:41]
+        self.len_ = fields[6]  # frame[42:45]
+        self.payload = fields[7]  # frame[46:].split(" ")[0]
+        self._len = int(len(self.payload) / 2)
+
+        try:
+            self.src, self.dst, *self._addrs = pkt_addrs(
+                " ".join(fields[i] for i in range(2, 5))  # frame[7:36]
+            )
+        except InvalidPacketError as exc:  # will be: InvalidAddrSetError
+            raise InvalidPacketError(f"Bad frame: invalid address set {exc}")
+
+        if len(self.payload) != int(self.len_) * 2:
+            raise InvalidPacketError(
+                f"Bad frame: invalid payload: "
+                f"len({self.payload}) is not int('{self.len_}' * 2))"
+            )
+
+        self._ctx_: Union[str, bool] = None  # type: ignore[assignment]
+        self._hdr_: str = None  # type: ignore[assignment]
+        self._idx_: Union[str, bool] = None  # type: ignore[assignment]
+
+        self._has_array_: bool = None  # type: ignore[assignment]
+        self._has_ctl_: bool = None  # type: ignore[assignment]  # TODO: remove
+        self._has_payload_: bool = None  # type: ignore[assignment]
+
+    @classmethod  # for internal use only
+    def _from_attrs(cls, verb, *addrs, code, payload, seqn=None):
+        """Create a frame from its attributes (args, kwargs)."""
+
+        seqn = seqn or "---"
+        len_ = f"{int(len(payload) / 2):03d}"
+
+        try:
+            return cls(" ".join(verb, seqn, *addrs, code, len_, payload))
+        except TypeError as exc:
+            raise InvalidPacketError(f"Bad frame: invalid attr: {exc}")
+
+    def _validate(self, *, strict_checking: bool = None) -> None:
+        """Validate the frame: it may be a cmd or a (response) pkt.
+
+        Raise an exception InvalidPacketError (InvalidAddrSetError) if it is not valid.
+        """
+
+        if (seqn := self._frame[3:6]) == "...":
+            raise InvalidPacketError(f"Bad frame: deprecated seqn: {seqn}")
+
+        if not strict_checking:
+            return
+
+        if len(self._frame[46:].split(" ")[0]) != int(self._frame[42:45]) * 2:
+            raise InvalidPacketError("Bad frame: payload length mismatch")
+
+        try:
+            self.src, self.dst, *self._addrs = pkt_addrs(self._frame[7:36])
+        except InvalidPacketError as exc:  # will be: InvalidAddrSetError
+            raise InvalidPacketError(f"Bad frame: invalid address set: {exc}")
+
+        if True:  # below is done at a higher layer
+            return
+
+        if (code := self._frame[37.41]) not in CODES_SCHEMA:
+            raise InvalidPacketError(f"Bad frame: unknown code: {code}")
+
+    def __repr__(self) -> str:
+        """Return a unambiguous string representation of this object."""
+        # repr(self) == repr(cls(repr(self)))
+
+        return " ".join(
+            (
+                self.verb,
+                self.seqn,
+                *(repr(a) for a in self._addrs),
+                self.code,
+                self.len_,
+                self.payload,
+            )
+        )
 
     def __str__(self) -> str:
         """Return a brief readable string representation of this object."""
-        return str() if self._frame is None else self._frame[4:]
+        # repr(self) == repr(cls(str(self)))
 
-    @property
-    def rssi(self) -> str:
-        return self._rssi
-
-    @property
-    def verb(self) -> str:
-        return self._verb
-
-    @property
-    def seqn(self) -> str:
-        return self._seqn
-
-    @property
-    def src(self) -> Address:
-        return self._src
-
-    @property
-    def dst(self) -> Address:
-        return self._dst
-
-    @property
-    def addrs(self) -> Tuple[Address, Address, Address]:
-        return self._addrs
-
-    @property
-    def code(self) -> str:
-        return self._code
-
-    @property
-    def len(self) -> int:  # payload length in 2-char bytes
-        return self._len
-
-    @property
-    def payload(self) -> str:
-        return self._payload
+        try:
+            return f"{self!r} # {self._hdr}"  # code|ver|device_id|context
+        except AttributeError as exc:
+            return f"{self!r} < {exc}"
 
     @property
     def _has_array(self) -> Optional[bool]:
@@ -199,19 +275,28 @@ class PacketBase:
         elif self.verb != I_ or self.code not in CODES_WITH_ARRAYS:
             self._has_array_ = False
 
-        elif self.len == CODES_WITH_ARRAYS[self.code][0]:  # NOTE: can be false -ves
-            self._has_array_ = self.code == _22C9  # 22C9 is a reliable exception
+        elif self._len == CODES_WITH_ARRAYS[self.code][0]:  # NOTE: can be false -ves
+            self._has_array_ = False
+            if (
+                self.code in (_22C9, _3150)  # only time 22C9 is seen
+                and self.src.type == DEV_TYPE_MAP.UFC
+                and self.dst is self.src
+                and self.payload[:1] != "F"
+            ):
+                self._has_array_ = True
 
         else:
-            _len = CODES_WITH_ARRAYS[self.code][0]
+            len_ = CODES_WITH_ARRAYS[self.code][0]
             assert (
-                self.len % _len == 0
-            ), f"{self} < array has length ({self.len}) that is not multiple of {_len}"
+                self._len % len_ == 0
+            ), f"{self} < array has length ({self._len}) that is not multiple of {len_}"
             assert (
-                self.src.type in ("12", "22") or self.src == self.dst  # DEX
+                self.src.type in (DEV_TYPE_MAP.DTS, DEV_TYPE_MAP.DT2)
+                or self.src == self.dst  # DEX
             ), f"{self} < array is from a non-controller (01)"
             assert (
-                self.src.type not in ("12", "22") or self.dst.id == NON_DEVICE_ID  # DEX
+                self.src.type not in (DEV_TYPE_MAP.DTS, DEV_TYPE_MAP.DT2)
+                or self.dst.id == NON_DEV_ADDR.id  # DEX
             ), f"{self} < array is from a non-controller (02)"
             self._has_array_ = True
 
@@ -237,15 +322,11 @@ class PacketBase:
 
         # TODO: handle RQ/RP to/from HGI/RFG, handle HVAC
 
-        # TODO: Not needed? Relies upon MSG layer in any case
-        # if getattr(self.src, "_is_controller", False):
-        #     # _LOGGER.info("HAS Controller (00)")
-        #     return True
-        # if getattr(self.dst, "_is_controller", False):
-        #     # _LOGGER.info("HAS controller (01)")
-        #     return True
-
-        if {self.src.type, self.dst.type} & {"01", "02", "23"}:  # DEX
+        if {self.src.type, self.dst.type} & {
+            DEV_TYPE_MAP.CTL,
+            DEV_TYPE_MAP.UFC,
+            DEV_TYPE_MAP.PRG,
+        }:  # DEX
             _LOGGER.debug(f"{self} # HAS controller (10)")
             self._has_ctl_ = True
 
@@ -253,26 +334,26 @@ class PacketBase:
         elif self.dst is self.src:  # (not needed?) & self.code == I_:
             _LOGGER.debug(
                 f"{self} < "
-                + ("HAS" if self.code in CODE_ONLY_FROM_CTL + [_31D9, _31DA] else "no")
+                + ("HAS" if self.code in CODES_ONLY_FROM_CTL + [_31D9, _31DA] else "no")
                 + " controller (20)"
             )
             self._has_ctl_ = any(
                 (
-                    self.code == _3B00 and self.payload[:2] == "FC",
-                    self.code in CODE_ONLY_FROM_CTL + [_31D9, _31DA],
+                    self.code == _3B00 and self.payload[:2] == FC,
+                    self.code in CODES_ONLY_FROM_CTL + [_31D9, _31DA],
                 )
             )
 
         #  I --- --:------ --:------ 10:050360 1FD4 003 002ABE # no ctl
         #  I 095 --:------ --:------ 12:126457 1F09 003 000BC2 # HAS ctl
         #  I --- --:------ --:------ 20:001473 31D9 003 000001 # ctl? (HVAC)
-        elif self.dst.id == NON_DEVICE_ID:
+        elif self.dst.id == NON_DEV_ADDR.id:
             _LOGGER.debug(f"{self} # HAS controller (21)")
-            self._has_ctl_ = self.src.type != "10"  # DEX
+            self._has_ctl_ = self.src.type != DEV_TYPE_MAP.OTB  # DEX
 
         #  I --- 10:037879 --:------ 12:228610 3150 002 0000   # HAS ctl
         #  I --- 04:029390 --:------ 12:126457 1060 003 01FF01 # HAS ctl
-        elif self.dst.type in ("12", "22"):  # DEX
+        elif self.dst.type in (DEV_TYPE_MAP.DTS, DEV_TYPE_MAP.DT2):  # DEX
             _LOGGER.debug(f"{self} # HAS controller (22)")
             self._has_ctl_ = True
 
@@ -291,7 +372,10 @@ class PacketBase:
         #  I --- 34:021943 63:262142 --:------ 10E0 038 000001C8380A01... # unknown
         #  I --- 32:168090 30:082155 --:------ 31E0 004 0000C800          # unknown
         if self._has_ctl_ is None:
-            if DEV_MODE and "18" not in (self.src.type, self.dst.type):  # DEX
+            if DEV_MODE and DEV_TYPE_MAP.HGI not in (
+                self.src.type,
+                self.dst.type,
+            ):  # DEX
                 _LOGGER.warning(f"{self} # has_ctl - undetermined (99)")
             self._has_ctl_ = False
 
@@ -309,10 +393,10 @@ class PacketBase:
 
         self._has_payload_ = not any(
             (
-                self.len == 1,
+                self._len == 1,
                 self.verb == RQ and self.code in RQ_NO_PAYLOAD,
-                self.verb == RQ and self.len == 2 and self.code != _0016,
-                # self.verb == RQ and self.len == 2 and self.code in (
+                self.verb == RQ and self._len == 2 and self.code != _0016,
+                # self.verb == RQ and self._len == 2 and self.code in (
                 #   _2309, _2349, _3EF1
                 # ),
             )
@@ -322,9 +406,9 @@ class PacketBase:
 
     def _force_has_array(self) -> None:
         self._has_array_ = True
-        self._ctx_ = None
-        self._hdr_ = None
-        self._idx_ = None
+        self._ctx_ = None  # type: ignore[assignment]
+        self._hdr_ = None  # type: ignore[assignment]
+        self._idx_ = None  # type: ignore[assignment]
 
     @property
     def _ctx(self) -> Union[str, bool]:  # incl. self._idx
@@ -334,10 +418,10 @@ class PacketBase:
         """
 
         if self._ctx_ is None:
-            if self.code in (_0005, _000C):  # zone_idx, zone_type (device_class)
+            if self.code in (_0005, _000C):  # zone_idx, zone_type (device_role)
                 self._ctx_ = self.payload[:4]
             elif self.code == _0404:  # zone_idx, frag_idx
-                self._ctx_ = self.payload[:2] + self.payload[10:12]
+                self._ctx_ = self._idx + self.payload[10:12]
             else:
                 self._ctx_ = self._idx
         return self._ctx_
@@ -350,12 +434,13 @@ class PacketBase:
         """
 
         if self._hdr_ is None:
+            self._hdr_ = "|".join((self.code, self.verb))  # HACK: RecursionError
             self._hdr_ = pkt_header(self)
         return self._hdr_
 
     @property
     def _idx(self) -> Union[str, bool]:
-        """Return the payload's index, if any (e.g. zone_idx, domain_id, or log_idx).
+        """Return the payload's index, if any (e.g. zone_idx, domain_id  or log_idx).
 
         Used to route a packet to the correct entity's (i.e. zone/domain) msg handler.
         """
@@ -366,7 +451,7 @@ class PacketBase:
 
 
 def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
-    """Return the payload's 2-byte context (e.g. zone_idx, log_idx, domain_id).
+    """Return the payload's 2-byte context (e.g. zone_idx, domain_id or log_idx).
 
     May return a 2-byte string (usu. pkt.payload[:2]), or:
     - False if there is no context at all
@@ -382,17 +467,23 @@ def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
         return pkt._has_array
 
     #  I --- 10:040239 01:223036 --:------ 0009 003 000000
-    if pkt.code == _0009 and pkt.src.type == "10":  # DEX
+    if pkt.code == _0009 and pkt.src.type == DEV_TYPE_MAP.OTB:  # DEX
         return False
 
     if pkt.code == _000C:  # zone_idx/domain_id (complex, payload[0:4])
-        if pkt.payload[2:4] in ("0D", "0E"):  # ("000D", _000E, "010E")
-            return "FA"
-        if pkt.payload[2:4] == "0F":  # ("000F", )
-            return "FC"
+        if pkt.payload[2:4] == DEV_ROLE_MAP.APP:  # "000F"
+            return FC
+        if pkt.payload[0:4] == f"01{DEV_ROLE_MAP.HTG}":  # "010E"
+            return F9
+        if pkt.payload[2:4] in (
+            DEV_ROLE_MAP.DHW,
+            DEV_ROLE_MAP.HTG,
+        ):  # "000D", "000E"
+            return FA
         return pkt.payload[:2]
 
-    # if pkt.code == _0404:  # TODO: is entry needed here, esp. for DHW?
+    if pkt.code == _0404:
+        return FA if pkt.payload[2:4] == "23" else pkt.payload[:2]
 
     if pkt.code == _0418:  # log_idx (payload[4:6])
         return pkt.payload[4:6]
@@ -408,11 +499,11 @@ def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
 
     # mutex 1/4, CODE_IDX_NONE: always returns False
     if pkt.code in CODE_IDX_NONE:  # returns False
-        if RAMSES_CODES[pkt.code].get(pkt.verb, "")[:3] == "^00" and (
+        if CODES_SCHEMA[pkt.code].get(pkt.verb, "")[:3] == "^00" and (
             pkt.payload[:2] != "00"
         ):
             raise InvalidPayloadError(
-                f"Packet idx is {pkt.payload[:2]}, but expecting no idx (00)"
+                f"Packet idx is {pkt.payload[:2]}, but expecting no idx (00) (0xAA)"
             )
         return False
 
@@ -421,14 +512,16 @@ def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
         return True  # excludes len==1 for 000A, 2309, 30C9
 
     # TODO: is this needed?: exceptions to CODE_IDX_SIMPLE
-    if pkt.payload[:2] in ("F8", "F9", "FA", "FC"):  # TODO: FB, FD
+    if pkt.payload[:2] in (F8, F9, FA, FC):  # TODO: FB, FD
         if pkt.code not in CODE_IDX_DOMAIN:
             raise InvalidPayloadError(
                 f"Packet idx is {pkt.payload[:2]}, but not expecting a domain id"
             )
         return pkt.payload[:2]
 
-    if pkt._has_ctl:  # risk of false -ves, TODO: pkt.src.type == "18" too?  # DEX
+    if (
+        pkt._has_ctl
+    ):  # risk of false -ves, TODO: pkt.src.type == DEV_TYPE_MAP.HGI too?  # DEX
         # 02:    22C9: would be picked up as an array, if len==1 counted
         # 03:    #  I 028 03:094242 --:------ 03:094242 30C9 003 010B22  # ctl
         # 12/22: 000A|1030|2309|30C9 from (addr0 --:), 1060|3150 (addr0 04:)
@@ -437,14 +530,15 @@ def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
 
     if pkt.payload[:2] != "00":
         raise InvalidPayloadError(
-            f"Packet idx is {pkt.payload[:2]}, but expecting no idx (00)"
-        )
+            f"Packet idx is {pkt.payload[:2]}, but expecting no idx (00) (0xAB)"
+        )  # TODO: add a test for this
 
     if pkt.code in CODE_IDX_SIMPLE:
-        return  # False  # TODO: return None (less precise) or risk false -ves?
+        return None  # False  # TODO: return None (less precise) or risk false -ves?
 
     # mutex 4/4, CODE_IDX_UNKNOWN: an unknown code
     _LOGGER.info(f"{pkt} # Unable to determine payload index (is probably OK)")
+    return None
 
 
 def pkt_header(pkt, rx_header=None) -> Optional[str]:  # NOTE: used in command.py
@@ -458,20 +552,22 @@ def pkt_header(pkt, rx_header=None) -> Optional[str]:  # NOTE: used in command.p
         #  W --- 01:145038 34:021943 --:------ 1FC9 006 00-2309-06368E  # wont know src until it arrives
         #  I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7
         if not rx_header:
-            device_id = NUL_DEVICE_ID if pkt.src == pkt.dst else pkt.dst.id
+            device_id = NUL_DEV_ADDR.id if pkt.src == pkt.dst else pkt.dst.id
             return "|".join((pkt.code, pkt.verb, device_id))
         if pkt.src == pkt.dst:  # and pkt.verb == I_:
             return "|".join((pkt.code, W_, pkt.src.id))
         if pkt.verb == W_:  # and pkt.src != pkt.dst:
-            return "|".join((pkt.code, I_, pkt.src.id))
-        return
+            return "|".join((pkt.code, I_, pkt.src.id))  # TODO: why not pkt.dst?
+        # if pkt.verb == RQ:  # and pkt.src != pkt.dst:  # TODO: this breaks things
+        #     return "|".join((pkt.code, RP, pkt.dst.id))
+        return None
 
-    addr = pkt.dst if pkt.src.type == "18" else pkt.src  # DEX
+    addr = pkt.dst if pkt.src.type == DEV_TYPE_MAP.HGI else pkt.src  # DEX
     if not rx_header:
         header = "|".join((pkt.code, pkt.verb, addr.id))
 
     elif pkt.verb in (I_, RP) or pkt.src == pkt.dst:  # announcements, etc.: no response
-        return
+        return None
 
     else:  # RQ/RP, or W/I
         header = "|".join((pkt.code, RP if pkt.verb == RQ else I_, addr.id))
