@@ -6,20 +6,46 @@
 Decode/process a message (payload into JSON).
 """
 
+# TODO:
+# long-format msg.__str__ - alias columns don't line up
+
 import logging
 import re
+from datetime import datetime as dt
 from datetime import timedelta as td
 from functools import lru_cache
-from typing import Any, Optional
+from typing import Optional
 
 from .address import Address
+from .const import (
+    DEV_TYPE_MAP,
+    SZ_ALIAS,
+    SZ_DHW_IDX,
+    SZ_DOMAIN_ID,
+    SZ_LOG_IDX,
+    SZ_UFH_IDX,
+    SZ_ZONE_IDX,
+    __dev_mode__,
+)
 from .exceptions import InvalidPacketError, InvalidPayloadError
 from .packet import fraction_expired
 from .parsers import PAYLOAD_PARSERS, parser_unknown
-from .ramses import CODE_IDX_COMPLEX, CODE_RQ_COMPLEX, RAMSES_CODES
+from .ramses import CODE_IDX_COMPLEX, CODES_SCHEMA, RQ_IDX_COMPLEX
 
-from .const import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
-from .const import (  # noqa: F401, isort: skip
+# skipcq: PY-W2000
+from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
+    I_,
+    RP,
+    RQ,
+    W_,
+    F9,
+    FA,
+    FC,
+    FF,
+)
+
+# skipcq: PY-W2000
+from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _0001,
     _0002,
     _0004,
@@ -49,6 +75,7 @@ from .const import (  # noqa: F401, isort: skip
     _10E0,
     _10E1,
     _1100,
+    _11F0,
     _1260,
     _1280,
     _1290,
@@ -62,6 +89,7 @@ from .const import (  # noqa: F401, isort: skip
     _1F09,
     _1F41,
     _1FC9,
+    _1FCA,
     _1FD0,
     _1FD4,
     _2249,
@@ -79,7 +107,9 @@ from .const import (  # noqa: F401, isort: skip
     _2420,
     _2D49,
     _2E04,
+    _2E10,
     _30C9,
+    _3110,
     _3120,
     _313F,
     _3150,
@@ -99,9 +129,7 @@ from .const import (  # noqa: F401, isort: skip
 
 __all__ = ["Message"]
 
-HVAC_ONLY_CODES = (_1298, _12A0, _12C8, _22F1, _22F3, _31D9, _31DA, _31E0)
-
-CODE_NAMES = {k: v["name"] for k, v in RAMSES_CODES.items()}
+CODE_NAMES = {k: v["name"] for k, v in CODES_SCHEMA.items()}
 
 MSG_FORMAT_10 = "|| {:10s} | {:10s} | {:2s} | {:16s} | {:^4s} || {}"
 MSG_FORMAT_18 = "|| {:18s} | {:18s} | {:2s} | {:16s} | {:^4s} || {}"
@@ -130,44 +158,53 @@ class Message:
 
         self.src = pkt.src
         self.dst = pkt.dst
-        self._addrs = pkt.addrs
+        self._addrs = pkt._addrs
 
-        self.dtm = pkt.dtm
+        self.dtm: dt = pkt.dtm
 
-        self.verb = pkt.verb
-        self.seqn = pkt.seqn
-        self.code = pkt.code
-        self.len = pkt.len
+        self.verb: str = pkt.verb
+        self.seqn: str = pkt.seqn
+        self.code: str = pkt.code
+        self.len: int = pkt._len
 
         self.code_name = CODE_NAMES.get(self.code, f"unknown_{self.code}")
 
         self._payload = self._validate(self._pkt.payload)  # ? raise InvalidPacketError
 
-        self._str = None
-        self._fraction_expired = None
-        self._is_fragment = None
+        self._str: str = None
+        self._fraction_expired: float = None
+        self._is_fragment: bool = None
 
     def __repr__(self) -> str:
         """Return an unambiguous string representation of this object."""
-        return repr(self._pkt)  # or str?
+        return str(self._pkt)  # repr or str?
 
     def __str__(self) -> str:
         """Return a brief readable string representation of this object."""
 
         def ctx(pkt) -> str:
             ctx = {True: "[..]", False: "", None: "??"}.get(pkt._ctx, pkt._ctx)
-            if not ctx and pkt.payload[:2] not in ("00", "FF"):
+            if not ctx and pkt.payload[:2] not in ("00", FF):
                 return f"({pkt.payload[:2]})"
             return ctx
 
         def display_name(addr: Address) -> str:  # TODO: needs caching
-            name = None
-            if self._gwy._include.get(addr.id):
+            """Return a friendly name for an Address, or a Device.
+
+            Use the alias, if one exists, or use a slug instead of a device type.
+            """
+
+            try:
                 if self._gwy.config.use_aliases:
-                    name = self._gwy._include[addr.id].get("alias")
-                if not name and (klass := self._gwy._include[addr.id].get("class")):
-                    name = f"{klass}:{addr.id[3:]}"
-            return (name or Address._friendly(addr.id))[:18]  # HACK
+                    return self._gwy._include[addr.id][SZ_ALIAS][:18]
+            except KeyError:
+                pass
+            try:
+                return f"{addr._SLUG}:{addr.id[3:]}"
+            except AttributeError:
+                pass
+
+            return Address._friendly(addr.id)
 
         if self._str is not None:
             return self._str
@@ -202,7 +239,7 @@ class Message:
         return self.dtm < other.dtm
 
     @property
-    def payload(self) -> Any:  # Any[dict, List[dict]]:
+    def payload(self):  # Union[dict, list[dict]]:
         """Return the payload."""
         return self._payload
 
@@ -223,7 +260,7 @@ class Message:
 
     @property
     def _idx(self) -> Optional[dict]:
-        """Return the zone_idx/domain_id of a message payload, if any.
+        """Return the domain_id/zone_idx/other_idx of a message payload, if any.
 
         Used to identify the zone/domain that a message applies to. Returns an empty
         dict if there is none such, or None if undetermined.
@@ -233,15 +270,17 @@ class Message:
 
         IDX_NAMES = {
             _0002: "other_idx",  # non-evohome: hometronics
-            _0418: "log_idx",  # can be 2 DHW zones per system
-            _10A0: "dhw_idx",  # can be 2 DHW zones per system
-            _22C9: "ufh_idx",  # UFH circuit
+            _0418: SZ_LOG_IDX,
+            _10A0: SZ_DHW_IDX,  # can be 2 DHW zones per system, albeit unusual
+            _1260: SZ_DHW_IDX,  # can be 2 DHW zones per system, albeit unusual
+            _1F41: SZ_DHW_IDX,  # can be 2 DHW zones per system, albeit unusual
+            _22C9: SZ_UFH_IDX,  # UFH circuit
             _2389: "other_idx",  # anachronistic
             _2D49: "other_idx",  # non-evohome: hometronics
             _31D9: "hvac_id",
             _31DA: "hvac_id",
             _3220: "msg_id",
-        }  # ALSO: "domain_id", "zone_idx"
+        }  # ALSO: SZ_DOMAIN_ID, SZ_ZONE_IDX
 
         if self._pkt._idx in (True, False) or self.code in CODE_IDX_COMPLEX:
             return {}  # above was: CODE_IDX_COMPLEX + [_3150]:
@@ -253,33 +292,33 @@ class Message:
 
         #  I --- 00:034798 --:------ 12:126457 2309 003 0201F4
         if not {self.src.type, self.dst.type} & {
-            "01",
-            "02",
-            "03",  # ?remove (see above, rare)
-            "12",
-            "18",
-            "22",
-            "23",
+            DEV_TYPE_MAP.CTL,
+            DEV_TYPE_MAP.UFC,
+            DEV_TYPE_MAP.HCW,  # ?remove (see above, rare)
+            DEV_TYPE_MAP.DTS,
+            DEV_TYPE_MAP.HGI,
+            DEV_TYPE_MAP.DT2,
+            DEV_TYPE_MAP.PRG,
         }:  # DEX
-            assert self._pkt._idx == "00", "What!! (00)"
+            assert self._pkt._idx == "00", "What!! (AA)"
             return {}
 
         #  I 035 --:------ --:------ 12:126457 30C9 003 017FFF
         if self.src.type == self.dst.type and self.src.type not in (
-            "01",
-            "02",
-            "03",  # ?remove (see above, rare)
-            "18",
-            "23",
+            DEV_TYPE_MAP.CTL,
+            DEV_TYPE_MAP.UFC,
+            DEV_TYPE_MAP.HCW,  # ?remove (see above, rare)
+            DEV_TYPE_MAP.HGI,
+            DEV_TYPE_MAP.PRG,
         ):  # DEX
-            assert self._pkt._idx == "00", "What!! (01)"
+            assert self._pkt._idx == "00", "What!! (AB)"
             return {}
 
         #  I --- 04:029362 --:------ 12:126457 3150 002 0162
         # if not getattr(self.src, "_is_controller", True) and not getattr(
         #     self.dst, "_is_controller", True
         # ):
-        #     assert self._pkt._idx == "00", "What!! (10)"
+        #     assert self._pkt._idx == "00", "What!! (BA)"
         #     return {}
 
         #  I --- 04:029362 --:------ 12:126457 3150 002 0162
@@ -287,17 +326,22 @@ class Message:
         #     getattr(self.src, "_is_controller", True)
         #     or getattr(self.dst, "_is_controller", True)
         # ):
-        #     assert self._pkt._idx == "00", "What!! (11)"
+        #     assert self._pkt._idx == "00", "What!! (BB)"
         #     return {}
 
         if self.src.type == self.dst.type and not getattr(
             self.src, "_is_controller", True
         ):  # DEX
-            assert self._pkt._idx == "00", "What!! (12)"
+            assert self._pkt._idx == "00", "What!! (BC)"
             return {}
 
+        # TODO: also 000C (but is a complex idx)
+        # TODO: also 3150 (when not domain, and will be array if so)
+        if self.code in (_000A, _2309) and self.src.type == DEV_TYPE_MAP.UFC:
+            return {IDX_NAMES[_22C9]: self._pkt._idx}
+
         index_name = IDX_NAMES.get(
-            self.code, "domain_id" if self._pkt._idx[:1] == "F" else "zone_idx"
+            self.code, SZ_DOMAIN_ID if self._pkt._idx[:1] == "F" else SZ_ZONE_IDX
         )
 
         return {index_name: self._pkt._idx}
@@ -338,8 +382,8 @@ class Message:
             ):
                 _logger = _LOGGER.info
             else:
-                _logger = _LOGGER.warning  # if DEV_MODE else _LOGGER.info  # TODO
-            _logger(f"{self._pkt} # has expired ({self._fraction_expired * 100:1.0f}%)")
+                _logger = _LOGGER.warning if DEV_MODE else _LOGGER.info
+            _logger(f"{self!r} # has expired ({self._fraction_expired * 100:1.0f}%)")
 
         # elif self._fraction_expired >= self.IS_EXPIRING:  # this could log multiple times
         #     _LOGGER.error("%s # is expiring", self._pkt)
@@ -377,8 +421,8 @@ class Message:
             # TODO: only accept invalid packets to/from HGI when flag raised
             _check_msg_payload(self, self._pkt.payload)  # ? InvalidPayloadError
 
-            if not self._has_payload or (
-                self.verb == RQ and self.code not in CODE_RQ_COMPLEX
+            if not self._has_payload and (
+                self.verb == RQ and self.code not in RQ_IDX_COMPLEX
             ):
                 # _LOGGER.error("%s", msg)
                 return {}
@@ -404,7 +448,7 @@ class Message:
             # beware: HGI80 can send 'odd' but parseable packets +/- get invalid reply
             (
                 _LOGGER.exception
-                if DEV_MODE and self.src.type != "18"  # DEX
+                if DEV_MODE and self.src.type != DEV_TYPE_MAP.HGI  # DEX
                 else _LOGGER.exception
             )("%s < %s", self._pkt, f"{exc.__class__.__name__}({exc})")
             raise InvalidPacketError(exc)
@@ -421,11 +465,11 @@ class Message:
 
 
 @lru_cache(maxsize=256)
-def re_compile_re_match(regex, string) -> bool:
+def re_compile_re_match(regex, string) -> bool:  # Optional[Match[Any]]
     # TODO: confirm this does speed things up
-    # Python has it's own caching of re.complile, _MAXCACHE = 512
+    # Python has its own caching of re.complile, _MAXCACHE = 512
     # https://github.com/python/cpython/blob/3.10/Lib/re.py
-    return re.compile(regex).match(string)
+    return re.compile(regex).match(string)  # type: ignore[return-value]
 
 
 def _check_msg_payload(msg: Message, payload) -> None:
@@ -438,11 +482,13 @@ def _check_msg_payload(msg: Message, payload) -> None:
     """
 
     try:
-        if msg.code not in RAMSES_CODES:
+        _ = repr(msg._pkt)  # HACK: ? raise InvalidPayloadError
+
+        if msg.code not in CODES_SCHEMA:
             raise InvalidPacketError(f"Unknown code: {msg.code}")
 
         try:
-            regex = RAMSES_CODES[msg.code][msg.verb]
+            regex = CODES_SCHEMA[msg.code][msg.verb]
         except KeyError:
             raise InvalidPacketError(f"Unknown verb/code pair: {msg.verb}/{msg.code}")
 
@@ -450,9 +496,12 @@ def _check_msg_payload(msg: Message, payload) -> None:
             raise InvalidPayloadError(f"Payload doesn't match '{regex}': {payload}")
 
     except InvalidPacketError as exc:  # incl. InvalidPayloadError
-        if "18" not in (msg.src.type, msg.dst.type):  # DEX, HGI80 can do what it likes
+        if DEV_TYPE_MAP.HGI not in (
+            msg.src.type,
+            msg.dst.type,
+        ):  # DEX, HGI80 can do what it likes
+            # _LOGGER.warning(f"{msg!r} < {exc}")
             raise exc  # TODO: messy - these msgs not ignore
-            _LOGGER.warning(f"{msg._pkt} < {exc}")
 
     # TODO: put this back, or leave it to the parser?
     # if msg.code == _3220:
