@@ -9,24 +9,24 @@ Decode/process a packet (packet that was received).
 import logging
 from datetime import datetime as dt
 from datetime import timedelta as td
-from typing import ByteString, Optional, Tuple
+from typing import Optional, Union
 
-from .address import pkt_addrs
+from .const import __dev_mode__
 from .exceptions import InvalidPacketError
-from .frame import PacketBase
+from .frame import Frame
 from .logger import getLogger
 from .opentherm import PARAMS_MSG_IDS, SCHEMA_MSG_IDS, STATUS_MSG_IDS
-from .ramses import EXPIRES, RAMSES_CODES
+from .ramses import CODES_SCHEMA, EXPIRES
 
+# skipcq: PY-W2000
 from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     I_,
-    MESSAGE_REGEX,
     RP,
     RQ,
     W_,
-    __dev_mode__,
 )
 
+# skipcq: PY-W2000
 from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _0001,
     _0002,
@@ -57,6 +57,7 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _10E0,
     _10E1,
     _1100,
+    _11F0,
     _1260,
     _1280,
     _1290,
@@ -70,6 +71,7 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _1F09,
     _1F41,
     _1FC9,
+    _1FCA,
     _1FD0,
     _1FD4,
     _2249,
@@ -87,7 +89,9 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _2420,
     _2D49,
     _2E04,
+    _2E10,
     _30C9,
+    _3110,
     _3120,
     _313F,
     _3150,
@@ -127,47 +131,77 @@ def fraction_expired(age, age_limit) -> float:
     return (age - _TD_SECONDS_003) / age_limit
 
 
-class Packet(PacketBase):
-    """The packet class; should trap/log all invalid PKTs appropriately."""
+class Packet(Frame):
+    """The Packet class (packets that were received).
+
+    They have a datetime (when received) an RSSI, and other meta-fields.
+    """
+
+    _dtm: dt
+    _rssi: str
 
     def __init__(self, gwy, dtm: dt, frame: str, **kwargs) -> None:
-        """Create a packet from a valid frame.
+        """Create a packet from a string (actually from f"{RSSI} {frame}").
 
-        Will raise InvalidPacketError (or InvalidAddrSetError) if it is invalid.
+        Will raise InvalidPacketError if it is invalid.
         """
-        super().__init__()
+
+        super().__init__(frame[4:])  # remove RSSI
 
         self._gwy = gwy
-        self._dtm = dtm
-        self._frame = frame
+        self._dtm: dt = dtm
 
-        self.comment = kwargs.get("comment")
-        self.error_text = kwargs.get("err_msg")
-        self.raw_frame = kwargs.get("raw_frame")
+        self._rssi: str = frame[0:3]
 
-        self._src, self._dst, self._addrs, self._len = self._validate(
-            self._frame[11:40]
-        )  # ? raise InvalidPacketError
+        self.comment: str = kwargs.get("comment", "")
+        self.error_text: str = kwargs.get("err_msg", "")
+        self.raw_frame: str = kwargs.get("raw_frame", "")
 
-        self._rssi = frame[0:3]
-        self._verb = frame[4:6]
-        self._seqn = frame[7:10]
-        self._code = frame[41:45]
-        self._payload = frame[50:]
-
-        self._timeout = None
+        self._timeout: Union[bool, float] = None  # track pkt expiry
 
         # if DEV_MODE:  # TODO: remove (is for testing only)
         #     _ = self._has_array
         #     _ = self._has_ctl
 
+        self._validate(strict_checking=False)
+
+    def _validate(self, *, strict_checking=None) -> None:
+        """Validate the packet, and parse the addresses if so (will log all packets).
+
+        Raise an exception InvalidPacketError (InvalidAddrSetError) if it is not valid.
+        """
+
+        try:
+            if self.error_text:
+                raise InvalidPacketError(self.error_text)
+
+            if not self._frame and self.comment:  # log null pkts only if has a comment
+                raise InvalidPacketError("Null packet")
+
+            super()._validate(strict_checking=strict_checking)  # no RSSI
+
+            _PKT_LOGGER.info("", extra=self.__dict__)  # the packet.log line
+
+        except InvalidPacketError as exc:  # incl. InvalidAddrSetError
+            if self._frame or self.error_text:
+                _PKT_LOGGER.warning("%s", exc, extra=self.__dict__)
+            raise exc
+
     def __repr__(self) -> str:
         """Return an unambiguous string representation of this object."""
-        hdr = f' # {self._hdr}{f" ({self._ctx})" if self._ctx else ""}'
         try:
-            return f"{self.dtm.isoformat(timespec='microseconds')} ... {self}{hdr}"
+            hdr = f' # {self._hdr}{f" ({self._ctx})" if self._ctx else ""}'
+        except (InvalidPacketError, NotImplementedError):
+            hdr = ""
+        try:
+            dtm = self.dtm.isoformat(timespec="microseconds")
         except AttributeError:
-            print()
+            dtm = dt.min.isoformat(timespec="microseconds")
+        return f"{dtm} ... {self}{hdr}"
+
+    def __str__(self) -> str:
+        """Return an brief readable string representation of this object."""
+        return super().__repr__()
 
     @property
     def dtm(self) -> dt:
@@ -179,7 +213,7 @@ class Packet(PacketBase):
         return self._frame[4:] == other._frame[4:]
 
     @staticmethod
-    def _partition(pkt_line: str) -> Tuple[str, str, str]:
+    def _partition(pkt_line: str) -> tuple[str, str, str]:  # map[str]
         """Partition a packet line into its three parts.
 
         Format: packet[ < parser-hint: ...][ * evofw3-err_msg][ # evofw3-comment]
@@ -188,10 +222,10 @@ class Packet(PacketBase):
         fragment, _, comment = pkt_line.partition("#")
         fragment, _, err_msg = fragment.partition("*")
         pkt_str, _, _ = fragment.partition("<")  # discard any parser hints
-        return map(str.strip, (pkt_str, err_msg, comment))
+        return map(str.strip, (pkt_str, err_msg, comment))  # type: ignore[return-value]
 
     @property
-    def _expired(self) -> float:
+    def _expired(self) -> Union[bool, float]:
         """Return the used fraction of the packet's 'normal' lifetime.
 
         A packet is 'expired' when >1.0 (should it be tombstoned when >2.0?). Returns
@@ -209,51 +243,21 @@ class Packet(PacketBase):
 
         return fraction_expired(self._gwy._dt_now() - self.dtm, self._timeout)
 
-    def _validate(self, addr_frag) -> None:
-        """Validate the packet, and parse the addresses if so (will log all packets).
-
-        Raise an exception InvalidPacketError (InvalidAddrSetError) if it is not valid.
-        """
-
-        try:
-            if self.error_text:
-                raise InvalidPacketError(self.error_text)
-
-            if not self._frame and self.comment:  # log null pkts only if has a comment
-                raise InvalidPacketError("Null packet")
-
-            if not MESSAGE_REGEX.match(self._frame):
-                raise InvalidPacketError("Invalid packet structure")
-
-            length = int(self._frame[46:49])
-            if len(self._frame[50:]) != length * 2:
-                raise InvalidPacketError("Invalid payload length")
-
-            src, dst, addrs = pkt_addrs(addr_frag)  # self._frame[11:40]
-
-            _PKT_LOGGER.info("", extra=self.__dict__)
-            return src, dst, addrs, length
-
-        except InvalidPacketError as exc:  # incl. InvalidAddrSetError
-            if self._frame or self.error_text:
-                _PKT_LOGGER.warning("%s", exc, extra=self.__dict__)
-            raise exc
-
     @classmethod
     def from_dict(cls, gwy, dtm: str, pkt_line: str):
-        """Constructor to create a packet from a saved state (a curated dict)."""
+        """Create a packet from a saved state (a curated dict)."""
         frame, _, comment = cls._partition(pkt_line)
         return cls(gwy, dt.fromisoformat(dtm), frame, comment=comment)
 
     @classmethod
     def from_file(cls, gwy, dtm: str, pkt_line: str):
-        """Constructor to create a packet from a log file line."""
+        """Create a packet from a log file line."""
         frame, err_msg, comment = cls._partition(pkt_line)
         return cls(gwy, dt.fromisoformat(dtm), frame, err_msg=err_msg, comment=comment)
 
     @classmethod
-    def from_port(cls, gwy, dtm: dt, pkt_line: str, raw_line: ByteString = None):
-        """Constructor to create a packet from a usb port (HGI80, evofw3)."""
+    def from_port(cls, gwy, dtm: dt, pkt_line: str, raw_line: bytes = None):
+        """Create a packet from a USB port (HGI80, evofw3)."""
         frame, err_msg, comment = cls._partition(pkt_line)
         return cls(
             gwy, dtm, frame, err_msg=err_msg, comment=comment, raw_frame=raw_line
@@ -270,10 +274,10 @@ def pkt_timeout(pkt) -> Optional[td]:  # NOTE: import OtbGateway ??
         return _TD_SECONDS_000
 
     if pkt.code in (_0005, _000C, _0404, _10E0):  # 0404 expired by 0006
-        return  # TODO: exclude/remove devices caused by corrupt ADDRs?
+        return None  # TODO: exclude/remove devices caused by corrupt ADDRs?
 
     if pkt.code == _1FC9 and pkt.verb == RP:
-        return  # TODO: check other verbs, they seem variable
+        return None  # TODO: check other verbs, they seem variable
 
     if pkt.code == _1F09:  # sends I /sync_cycle
         # can't do better than 300s with reading the payload
@@ -289,7 +293,7 @@ def pkt_timeout(pkt) -> Optional[td]:  # NOTE: import OtbGateway ??
         # if pkt.payload[4:6] in WRITE_MSG_IDS and Write-Data:  # TODO
         #     return _TD_SECONDS_003
         if pkt.payload[4:6] in SCHEMA_MSG_IDS:
-            return  # SCHEMA_MSG_IDS[pkt.payload[4:6]]
+            return None  # SCHEMA_MSG_IDS[pkt.payload[4:6]]
         if pkt.payload[4:6] in PARAMS_MSG_IDS:
             return PARAMS_MSG_IDS[pkt.payload[4:6]]
         if pkt.payload[4:6] in STATUS_MSG_IDS:
@@ -299,7 +303,7 @@ def pkt_timeout(pkt) -> Optional[td]:  # NOTE: import OtbGateway ??
     # if pkt.code in (_3B00, _3EF0, ):  # TODO: 0008, 3EF0, 3EF1
     #     return td(minutes=6.7)  # TODO: WIP
 
-    if (code := RAMSES_CODES.get(pkt.code)) and EXPIRES in code:
-        return RAMSES_CODES[pkt.code][EXPIRES]
+    if (code := CODES_SCHEMA.get(pkt.code)) and EXPIRES in code:
+        return CODES_SCHEMA[pkt.code][EXPIRES]
 
     return _TD_MINUTES_060
